@@ -3,6 +3,7 @@ import os
 os.environ['TRANSFORMERS_CACHE'] = 'cache'  # Must be before transformers import.
 import pandas as pd
 import torch
+import tqdm
 import transformers
 from sklearn.model_selection import train_test_split
 
@@ -10,6 +11,8 @@ from .task import Task
 
 class GenerateTask(Task):
     def __init__(self, train, val, test):
+        self.num_steps = 2 if train is None else 3
+        print(f'\n(1/{self.num_steps}) PROCESSING DATA...\n')
         (
             self.train_dataset,
             self.val_dataset,
@@ -21,26 +24,38 @@ class GenerateTask(Task):
     def _load_data(self, train, val, test):
         self.tokenizer = transformers.T5Tokenizer.from_pretrained('google/flan-t5-base', cache_dir='cache/flan-t5')
         
+        print('  - Data Processing Step 1 of 3...')
         train_tokens = None
-        if train:
-            train_tokens = self.tokenizer([t[0] for t in train], return_tensors='pt', padding=True)
-            train_labels = self.tokenizer([t[1] for t in train], return_tensors='pt', padding=True)
-            train_tokens['labels'] = train_labels['input_ids']
+        if train is not None:
+            with tqdm.tqdm(total=2) as pbar:
+                train_tokens = self.tokenizer(train.prompt.tolist(), return_tensors='pt', padding=True)
+                pbar.update()
+                train_labels = self.tokenizer(train.label.tolist(), return_tensors='pt', padding=True)
+                train_tokens['labels'] = train_labels['input_ids']
+                pbar.update()
         
+        print('  - Data Processing Step 2 of 3...')
         val_tokens = None
-        if val:
-            val_tokens = self.tokenizer([v[0] for v in val], return_tensors='pt', padding=True)
-            val_labels = self.tokenizer([v[1] for v in val], return_tensors='pt', padding=True)
-            val_tokens['labels'] = val_labels['input_ids']
+        if val is not None:
+            with tqdm.tqdm(total=2) as pbar:
+                val_tokens = self.tokenizer(val.prompt.tolist(), return_tensors='pt', padding=True)
+                pbar.update()
+                val_labels = self.tokenizer(val.label.tolist(), return_tensors='pt', padding=True)
+                val_tokens['labels'] = val_labels['input_ids']
+                pbar.update()
 
-        test_tokens = self.tokenizer(test, return_tensors='pt', padding=True)
+        print('  - Data Processing Step 3 of 3...')
+        with tqdm.tqdm(total=1) as pbar:
+            test_tokens = self.tokenizer(test.prompt.tolist(), return_tensors='pt', padding=True)
+            pbar.update()
 
-        return train_tokens, val_tokens, [v[1] for v in val], test_tokens
+        return train_tokens, val_tokens, (val.label.tolist() if val is not None else None), test_tokens
 
     def _load_model(self):
         return transformers.T5ForConditionalGeneration.from_pretrained('google/flan-t5-base', cache_dir='cache/flan-t5')
 
     def train(self):
+        print(f'\n(2/{self.num_steps}) LEARNING...\n')
         class FT5Dataset(torch.utils.data.Dataset):
             def __init__(self, data):
                 self.data = data
@@ -53,7 +68,7 @@ class GenerateTask(Task):
 
         args = transformers.Seq2SeqTrainingArguments(
             'cache/flan-t5',
-            num_train_epochs=15,
+            num_train_epochs=10,
             evaluation_strategy='epoch',
             save_strategy='epoch',
             load_best_model_at_end=True,
@@ -70,6 +85,7 @@ class GenerateTask(Task):
         trainer.train()
 
     def evaluate(self):
+        print('\nGETTING METRICS...\n')
         # Remove 'labels' key from validation dataset for prediction.
         dummy_val = self.val_dataset.copy()
         del dummy_val['labels']
@@ -88,33 +104,39 @@ class GenerateTask(Task):
         return metrics
 
     def predict(self):
-        outputs = self.model.to(self.test_dataset['input_ids'].device).generate(**self.test_dataset)
+        print(f'\n({self.num_steps}/{self.num_steps}) CREATING TAILWIZ LABELS...\n')
+        with tqdm.tqdm(total=1) as pbar:
+            outputs = self.model.to(self.test_dataset['input_ids'].device).generate(**self.test_dataset)
+            pbar.update()
         return self.tokenizer.batch_decode(outputs, skip_special_tokens=True)
 
 
-def generate(text_to_label, prelabeled_text=None, output_metrics=False):
-    if isinstance(text_to_label, pd.DataFrame):
-        assert 'prompt' in text_to_label.columns, \
-            'Make sure the prompt column in your pandas DataFrame is named "prompt".'
-        text_to_label = text_to_label.prompt.tolist()
-    if isinstance(prelabeled_text, pd.DataFrame):
-        assert 'prompt' in prelabeled_text.columns and 'label' in prelabeled_text.columns, \
+def generate(to_generate, labeled_examples=None, output_metrics=False):
+    assert isinstance(to_generate, pd.DataFrame), 'Make sure you are passing in pandas DataFrames.'
+    assert 'prompt' in to_generate.columns, \
+        'Make sure the prompt column in your pandas DataFrame is named "prompt".'
+    if labeled_examples is not None:
+        assert isinstance(labeled_examples, pd.DataFrame), 'Make sure you are passing in pandas DataFrames.'
+        assert 'prompt' in labeled_examples.columns and 'label' in labeled_examples.columns, \
             'Make sure the prompt column in your pandas DataFrame is named "prompt" and the label column is named "label".'
-        prelabeled_text = list(zip(prelabeled_text.prompt.tolist(), prelabeled_text.label.tolist()))
     if output_metrics:
-        assert prelabeled_text is not None, 'In order to output an estimate of performance with output_metrics, prelabeled_text must be provided.'
+        assert labeled_examples is not None, 'In order to output an estimate of performance with output_metrics, labeled_examples must be provided.'
 
-    if prelabeled_text is None:
-        generate_task_out = GenerateTask([], [], text_to_label)
-        results = generate_task_out.predict()
+    if labeled_examples is None:
+        generate_task_out = GenerateTask(None, None, to_generate)
+        pred_results = generate_task_out.predict()
     else:
-        assert len(prelabeled_text) >= 2, 'At least 2 rows of prelabeled data must be given.'
-        train, val = train_test_split(prelabeled_text, test_size=0.2)
-        generate_task_out = GenerateTask(train, val, text_to_label)
+        assert len(labeled_examples) >= 2, 'At least 2 rows of prelabeled data must be given.'
+        train, val = train_test_split(labeled_examples, test_size=0.2)
+        generate_task_out = GenerateTask(train, val, to_generate)
         generate_task_out.train()
-        results = generate_task_out.predict()
+        pred_results = generate_task_out.predict()
+    
+    results = to_generate.copy()
+    results['tailwiz_label'] = pred_results
 
     if output_metrics:
         metrics = generate_task_out.evaluate()
 
+    print('\nDONE')
     return (results, metrics) if output_metrics else results
